'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPluggableMiddleware;

var _lodash = require('lodash.memoize');

var _lodash2 = _interopRequireDefault(_lodash);

var _checkForNonFunctions = require('./checkForNonFunctions');

var _checkForNonFunctions2 = _interopRequireDefault(_checkForNonFunctions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This is used for hot reloading redux middleware.
 *
 * let hmrMiddleware = createPluggableMiddleware(myMiddleware);
 * // hmrMiddleware can be used with applyMiddleware() just like anything else.
 * // but later if you need to replace it you can do:
 * hmrMiddleware.replaceMiddleware(myNewMiddleware);
 */
function createPluggableMiddleware(middleware) {
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions2.default)(middleware, 'middleware');

  var result = function result(store) {
    return function (next) {
      var dispatch = (0, _lodash2.default)(function (middleware) {
        return middleware ? middleware(store)(next) : next;
      });
      return function (action) {
        return dispatch(middleware)(action);
      };
    };
  };
  result.replaceMiddleware = function (nextMiddleware) {
    if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions2.default)(nextMiddleware, 'nextMiddleware');
    middleware = nextMiddleware;
  };
  return result;
}